<?php
/**
 * DBCart 商城系统
 *
 * ==========================================================================
 * @link      https://www.dbcart.cn/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbcart.cn/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Admin\Controller;

use Admin\Data\Common;
use Admin\Entity\Delivery;
use Admin\Entity\DeliveryContent;
use Admin\Entity\DeliveryIndividuation;
use Admin\Entity\Region;
use Admin\Form\DeliveryForm;
use Admin\Form\DeliveryIndividuationForm;
use Admin\Service\DeliveryContentManager;
use Admin\Service\DeliveryIndividuationManager;
use Admin\Service\DeliveryManager;
use Doctrine\ORM\EntityManager;
use Goods\Entity\GoodsClass;
use Laminas\Mvc\I18n\Translator;
use Laminas\View\Model\JsonModel;

class DeliveryController extends AdminActionController
{
    private $translator;
    private $entityManager;
    private $deliveryManager;
    private $deliveryIndividuationManager;
    private $deliveryContentManager;

    public function __construct(
        Translator      $translator,
        EntityManager   $entityManager,
        DeliveryManager $deliveryManager,
        DeliveryIndividuationManager $deliveryIndividuationManager,
        DeliveryContentManager $deliveryContentManager
    )
    {
        $this->translator       = $translator;
        $this->entityManager    = $entityManager;
        $this->deliveryManager  = $deliveryManager;
        $this->deliveryIndividuationManager = $deliveryIndividuationManager;
        $this->deliveryContentManager = $deliveryContentManager;
    }

    /**
     * 配送列表
     * @return array
     */
    public function indexAction()
    {
        $deliveryList = $this->entityManager->getRepository(Delivery::class)->findDeliveryList();
        return ['deliveryList' => $deliveryList];
    }

    /**
     * 添加配送方式
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function addAction()
    {
        $form = new DeliveryForm();
        $form->get('cashOnDelivery')->setValueOptions(Common::cashOnDeliveryState($this->translator));
        $form->get('deliveryNameCode')->setValueOptions($this->adminCommon()->deliveryNameCodeOptions());

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();

                try {
                    $deliveryInfo = $this->deliveryManager->addDelivery($data);
                    $this->deliveryContentManager->addOrEditDeliveryContent($data, $deliveryInfo->getDeliveryId());

                    $this->adminCommon()->addOperLog(sprintf($this->translator->translate('配送方式 %s 添加成功!'), $data['deliveryName'.SHOP_FONT_LANGUAGE]), $this->translator->translate('配送设置'));

                } catch (\Exception $e) {
                    $this->flashMessenger()->addWarningMessage($this->translator->translate('配送方式添加失败!'));
                }

                return $this->redirect()->toRoute('delivery');
            }
        }

        return ['form' => $form];
    }

    /**
     * 编辑配送方式
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function editAction()
    {
        $deliveryId = (int) $this->params()->fromRoute('id', -1);
        $deliveryInfo = $this->entityManager->getRepository(Delivery::class)->findOneByDeliveryId($deliveryId);
        if($deliveryInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该配送方式不存在!'));
            return $this->redirect()->toRoute('delivery');
        }

        $form = new DeliveryForm();
        $form->get('cashOnDelivery')->setValueOptions(Common::cashOnDeliveryState($this->translator));
        $form->get('deliveryNameCode')->setValueOptions($this->adminCommon()->deliveryNameCodeOptions());

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();

                try {
                    $this->deliveryManager->editDelivery($data, $deliveryInfo);
                    $this->deliveryContentManager->addOrEditDeliveryContent($data, $deliveryId);

                    $this->adminCommon()->addOperLog(sprintf($this->translator->translate('配送方式 %s 编辑成功!'), $data['deliveryName'.SHOP_FONT_LANGUAGE]), $this->translator->translate('配送设置'));
                } catch (\Exception $e) {
                    $this->flashMessenger()->addWarningMessage($this->translator->translate('配送方式编辑失败!'));
                }
                return $this->redirect()->toRoute('delivery');
            }
        } else $form->setData($deliveryInfo->valuesArray());

        $deliveryContentList = $this->entityManager->getRepository(DeliveryContent::class)->findBy(['deliveryId' => $deliveryId]);
        foreach ($deliveryContentList as $contentValue) {
            if (!in_array($contentValue->getLanguageCode(), SHOP_LANGUAGE)) continue;
            $form->get('deliveryName'.$contentValue->getLanguageCode())->setValue($contentValue->getDeliveryName());
            $form->get('deliveryInfo'.$contentValue->getLanguageCode())->setValue($contentValue->getDeliveryInfo());
        }

        return [
            'form'          => $form,
            'delivery'      => $deliveryInfo
        ];
    }

    /**
     * 删除配送方式
     * @return \Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function deleteAction()
    {
        if(!$this->adminCommon()->validatorCsrf()) return $this->redirect()->toRoute('delivery');

        $deliveryId = (int) $this->params()->fromRoute('id', -1);
        $deliveryInfo = $this->entityManager->getRepository(Delivery::class)->findOneByDeliveryId($deliveryId);
        if($deliveryInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该配送方式不存在!'));
            return $this->redirect()->toRoute('delivery');
        }
        $deliveryContent = $this->entityManager->getRepository(DeliveryContent::class)->findOneBy(['deliveryId' => $deliveryId, 'languageCode' => SHOP_FONT_LANGUAGE]);

        $this->deliveryManager->deleteDelivery($deliveryInfo);
        $this->deliveryContentManager->deleteDeliveryContent($deliveryId);

        $this->adminCommon()->addOperLog(sprintf($this->translator->translate('配送方式 %s 删除成功!'), $deliveryContent->getDeliveryName()), $this->translator->translate('配送设置'));

        return $this->adminCommon()->toReferer();
    }
}